"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildServer = void 0;
const tslib_1 = require("tslib");
const fs = require("fs");
const post_rendering_hook_1 = require("./hooks/post-rendering-hook");
const load_esm_1 = require("./utils/load-esm");
function buildServer(options, nitroConfig) {
    var _a, _b, _c, _d, _e;
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { createNitro, build, prepare, copyPublicAssets, prerender } = yield (0, load_esm_1.loadEsmModule)('nitropack');
        const nitro = yield createNitro(Object.assign({ dev: false, preset: process.env['BUILD_PRESET'] }, nitroConfig));
        if ((_a = options === null || options === void 0 ? void 0 : options.prerender) === null || _a === void 0 ? void 0 : _a.postRenderingHooks) {
            (0, post_rendering_hook_1.addPostRenderingHooks)(nitro, options.prerender.postRenderingHooks);
        }
        yield prepare(nitro);
        yield copyPublicAssets(nitro);
        if (((_b = nitroConfig === null || nitroConfig === void 0 ? void 0 : nitroConfig.prerender) === null || _b === void 0 ? void 0 : _b.routes) &&
            ((_c = nitroConfig === null || nitroConfig === void 0 ? void 0 : nitroConfig.prerender) === null || _c === void 0 ? void 0 : _c.routes.find((route) => route === '/'))) {
            // Remove the root index.html so it can be replaced with the prerendered version
            if (fs.existsSync(`${(_d = nitroConfig === null || nitroConfig === void 0 ? void 0 : nitroConfig.output) === null || _d === void 0 ? void 0 : _d.publicDir}/index.html`)) {
                fs.unlinkSync(`${(_e = nitroConfig === null || nitroConfig === void 0 ? void 0 : nitroConfig.output) === null || _e === void 0 ? void 0 : _e.publicDir}/index.html`);
            }
        }
        console.log(`Prerendering static pages...`);
        yield prerender(nitro);
        if (!(options === null || options === void 0 ? void 0 : options.static)) {
            console.log('Building Server...');
            yield build(nitro);
        }
        yield nitro.close();
    });
}
exports.buildServer = buildServer;
//# sourceMappingURL=build-server.js.map